function results = ictkm(dinit, dataObj, options)
% -------------------------------------------------------------------------
% Perform dictionary learning via IcTKM. 
% -------------------------------------------------------------------------
% Input:
% -------------------------------------------------------------------------
%   - 'dinit': input dictionary
%   - 'dataObj': training data object which inherits from an abstract class
%                refer to 'ictkm_data.m'
%   - 'options' structure with the following fields:
%        - 'numIter': Number of IcTKM iterations
%        - 'd': ambient dimension
%        - 'K': number of atoms to learn
%        - 'm': JL embedding dimension with m < d
%        - 'FJLT': JL transformation type. Following values are available:
%               - 'PartialFFT': uses the discrete Fourier matrix.
%               - 'PartialDCT': uses the discrete cosine matrix.
%               - 'PartialCirculant': uses a circulant matrix construction 
%                  based on the Rademacher vector.
%        - 'randomSeed': seed for the random number generator used for 
%                        drawing the JL embedding.
%        - 'noSketching': disables point embedding
%        - 'runMode': Sets run mode. Following values are available: 
%                   - 'parallel': speedup computations by parallelizing the 
%                      dictionary update formula. Uses the parallel toolbox 
%                      to fully utilize multicore architecture resources. 
%                      Note: Parallel runmode option requires a working 
%                      parallel toolbox installation.
%                   - 'sequential': dictionary update formula is computed 
%                     sequentially.
%        - 'useGPU': speed-up vectorized calculations by using a CUDA GPU.
%        - 'verbose': set to 1 to display the algorithm's status on the 
%                     console.
% -------------------------------------------------------------------------
% Output:
% -------------------------------------------------------------------------
%   - 'results' structure with the following fields:
%        - 'rdico': Output (learned) dictionary.
%        - 'avgTimePerIter': average CPU time per iteration in seconds.
%        - 'timePerIter': vector with CPU time of each iteration in seconds.
%        - 'runTime': total algorithm CPU time in seconds.
%
%                If 'dataObj' is of type 'IcTKMSyntheticData' the following 
%                additional fields are returned:
%
%        - 'rcorrIter': correlations between oracle and learned atoms.
%        - 'recovery_percent': percent of recovered atoms.
%        - 'approx_error': recovered dictionary approximation error
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compressed Dictionary Learning Toolbox
% Copyright (C) Flavio Teixeira 2018
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, 
% Boston, MA  02110-1301, USA.    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

options = ictkm_set_defaults(options, dataObj);
[dold, options, results] = ictkm_init(options, dinit, dataObj);

%-------------------------------------------------
% IcTKM main loop
%-------------------------------------------------

for it=1:options.numIter
    tic;
    
    results.iteration = it;
    ictkm_update_display(options, dataObj, results);
    
    options = ictkm_redraw_embedding(options);
    
    if strcmp(options.runMode, 'parallel')
        dnew = ictkm_core_parfor(dold, options, results, dataObj);
    elseif strcmp(options.runMode, 'sequential')
        dnew = ictkm_core(dold, options, dataObj);
    end   
    
    %-------------------------------------------------
    % Update dictionary with output from the IcTKMCore
    %-------------------------------------------------
    scale = sum(dnew.*dnew);
    % redraw atoms that are not used
    iszero=find(scale < 1e-9);
    if options.useGPU
        dnew(:,iszero) = randn(options.d, length(iszero), ...
            'double', 'gpuArray');
    else
        dnew(:,iszero) = randn( options.d, length(iszero) );
    end    
    
    scale(iszero) = sum( dnew(:,iszero).*dnew(:,iszero) );
    % normalize
    if options.useGPU
        dnew = dnew * spdiags( 1./sqrt(scale'), 0, options.K, options.K);
    else
        dnew = dnew * diag( 1./sqrt(scale) );
    end    
    %-------------------------------------------------    
    results.iterationTime = toc;
    results = ictkm_update_results(results, options, dataObj, dnew);
    
    dold = dnew;
               
    if results.haltIteration
        break;
    end
end

if options.useGPU
    % Retrieve learned dictionary from GPU to workspace.
    rdico = gather(dnew);
else
    rdico = dnew;
end

% Save results to structure
results.rdico = rdico;
results.dataObj = dataObj;

end