function dnew = ictkm_core(dold, opts, dataObj)
% -------------------------------------------------------------------------
% Runs the IcTKM dictionary updates in a sequential way. 
% -------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compressed Dictionary Learning Toolbox
% Copyright (C) Flavio Teixeira 2018
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, 
% Boston, MA  02110-1301, USA.    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if opts.useGPU
    dnew = zeros(opts.d,opts.K, 'double', 'gpuArray');
else
    dnew = zeros(opts.d,opts.K);
end

Sktchdold = dold;
if ~opts.noSketching
    partial = randperm(opts.d);
    if strcmp(opts.FJLTtype,'PartialFFT')
        Sktchdold = fft(opts.randSignMat.*Sktchdold);
    elseif strcmp(opts.FJLTtype,'PartialDCT')
        Sktchdold = rdct(opts.randSignMat.*Sktchdold,opts);
    elseif strcmp(opts.FJLTtype,'PartialCirculant')
        Sktchdold = ifft2( opts.circulConvoMatrix * ...
            fft2(opts.randSignMat.*Sktchdold) );
    end
    
    % Samples m rows at random from Sktchdold.
    Sktchdold = Sktchdold(partial(1:opts.m),:);
end

if (opts.verbose)
    fprintf('\t\t');
    pb_ = cmd_line_progress_bar('Current iter.:      ');
end

% Precompute gram to speed up computation of residuals
gram=dold'*dold;

while dataObj.n < dataObj.N
    if (opts.verbose)
        pb_.print(dataObj.n, dataObj.N);
    end
    
    % Loop through (and generate) each training
    % signal, and process it on-the-fly.
    [dataObj, data_] = dataObj.GetDataPoint();
    
    SktchData_ = data_;
    if ~opts.noSketching
        %------------------------------------------------------------------
        % Sketch data
        %------------------------------------------------------------------
        if strcmp(opts.FJLTtype,'PartialFFT')
            SktchData_ = fft(opts.randSignCol.*SktchData_);
        elseif strcmp(opts.FJLTtype,'PartialDCT')
            SktchData_ = rdct(opts.randSignCol.*SktchData_,opts);
        elseif strcmp(opts.FJLTtype,'PartialCirculant')
            SktchData_ = ifft( opts.circulConvoVector.* ...
                fft(opts.randSignCol.*SktchData_) );
        end
        
        % Samples m entries at random from SktchData_.
        SktchData_ = SktchData_( partial(1:opts.m) );
    end
    
    %----------------------------------------------------------------------
    % Compressed thresholding
    %----------------------------------------------------------------------
    sktchIp_n = Sktchdold'*SktchData_;
    absSktchIp_n = abs(sktchIp_n);
    [~,I] = sort(absSktchIp_n,1,'descend');
    In_S = I(1:dataObj.S);
    
    %----------------------------------------------------------------------
    
    if opts.noSketching
        % No need to recompute the inner product
        ipn_S = sktchIp_n(In_S);
        absIpn_S = absSktchIp_n(In_S);
    else
        ipn_S = dold(:,In_S)'*data_;
        absIpn_S = abs(ipn_S);
    end
    signIpn_S = sign(ipn_S);
    
    residual = data_ - dold(:,In_S) * (gram(In_S,In_S)\ipn_S);
    %residual = data_ - dold(:,In_S) * pinv( gram(In_S,In_S) ) * ipn_S;
    
    dnew(:,In_S) = dnew(:,In_S) + real(residual*signIpn_S');
    dnew(:,In_S) = dnew(:,In_S) + dold(:,In_S)*diag(absIpn_S);
end

end