function dnew = ictkm_core_parfor(dold, opts, results, dataObj)
% -------------------------------------------------------------------------
% Runs the IcTKM dictionary updates in a parallel way. 
% -------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compressed Dictionary Learning Toolbox
% Copyright (C) Flavio Teixeira 2018
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, 
% Boston, MA  02110-1301, USA.    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Precompute gram to speed up computation of residuals
gram=dold'*dold;

%--------------------------------------------------------------------------
% Avoid broadcasting opts/results structs by making local copies
%--------------------------------------------------------------------------
m = opts.m;
noSketching = opts.noSketching;
FJLTtype = [];
if ~noSketching
    FJLTtype = opts.FJLTtype;
end
S = dataObj.S;
d = opts.d;
K = opts.K;
it = results.iteration; 
useSeed = opts.useSeed;
if useSeed
    randomSeed = opts.randomSeed;
else
    % Dummy. If this is not declared parfor will complain.
    randomSeed = []; 
end
useGPU = opts.useGPU;

partial = [];
randSignCol = [];
circulConvoVector = [];

%--------------------------------------------------------------------------
% Sketch dictionary
%--------------------------------------------------------------------------
Sktchdold = dold;
if ~noSketching    
    partial = randperm(opts.d);    
    if strcmp(FJLTtype,'PartialFFT')
        Sktchdold = fft(opts.randSignMat.*Sktchdold);                
    elseif strcmp(FJLTtype,'PartialDCT')
        Sktchdold = rdct(opts.randSignMat.*Sktchdold,opts);
    elseif strcmp(FJLTtype,'PartialCirculant')
        Sktchdold = ifft2( opts.circulConvoMatrix * ...
            fft2(opts.randSignMat.*Sktchdold) );        
    end
    
    % Samples m rows at random from Sktchdold.
    Sktchdold = Sktchdold(partial(1:opts.m),:);
    
    % Make local copies
    randSignCol = opts.randSignCol;
    if strcmp(FJLTtype,'PartialCirculant')
        circulConvoVector = opts.circulConvoVector;
    end
end


%--------------------------------------------------------------------------
% Initialize dictionary update
%--------------------------------------------------------------------------
if useGPU
    dnew = zeros(d, K, 'double', 'gpuArray');
else
    dnew = zeros(d, K);
end

%--------------------------------------------------------------------------
% Parallelize dictionary updates
%--------------------------------------------------------------------------
parfor n=1:dataObj.N       
    %----------------------------------------------------------------------
    % Set up random number generator of each worker
    %----------------------------------------------------------------------
    if useSeed
        sc = RandStream('CombRecursive','NormalTransform','Inversion',...
            'Seed', randomSeed + n*it);
        RandStream.setGlobalStream(sc);
        if useGPU  
            % GPU has its own random generator
            sg = parallel.gpu.RandStream('CombRecursive',...
                'NormalTransform','Inversion','Seed', randomSeed + n*it);
            parallel.gpu.RandStream.setGlobalStream(sg);
        end
    end
    %----------------------------------------------------------------------
    % Make local copies of pertaining variables 
    % to the workers
    %----------------------------------------------------------------------
    % Removes the need to broadcast the variables
    % below by making local copies to each worker
     gramPar = gram;    
     dold_ = dold;          
%     noSketching_ = noSketching;
%     useGPU_ = useGPU;
     partial_ = partial;
%     FJLTtype_ = [];
%     if ~noSketching_
%         FJLTtype_ = FJLTtype; 
%     end    
% 
     dataObj_ = dataObj;               
    %----------------------------------------------------------------------
    % Generate each training signal, and process it on-the-fly.
    [~, data_n_] = dataObj_.GetDataPoint();     
    
    %----------------------------------------------------------------------    
    SktchData_n = data_n_;    
    if ~noSketching
        %------------------------------------------------------------------
        % Sketch data
        %------------------------------------------------------------------
        if strcmp(FJLTtype,'PartialFFT')
            SktchData_n = fft(randSignCol.*SktchData_n);            
        elseif strcmp(FJLTtype,'PartialDCT')
            SktchData_n = rdct(randSignCol.*SktchData_n,opts);
        elseif strcmp(FJLTtype,'PartialCirculant')
            SktchData_n = ifft( circulConvoVector.* ...
                fft(randSignCol.*SktchData_n) );
        end
        
        % Samples m entries at random from SktchData_.
        SktchData_n = SktchData_n( partial_(1:m) );
    end
   
    %----------------------------------------------------------------------
    % Compressed thresholding
    %----------------------------------------------------------------------
    sktchIp_n = Sktchdold'*SktchData_n;
    absSktchIp_n = abs(sktchIp_n);
    [~,I] = sort(absSktchIp_n,1,'descend');    
    if useGPU
        In_S = gpuArray( I(1:S) );
        I_d = gpuArray.colon(1, d);
    else
        In_S = I(1:S);
        I_d = 1:d;
    end    
    
    %----------------------------------------------------------------------
    if noSketching        
        % No need to recompute the inner product
        ipn_S = sktchIp_n(In_S);   
        absIpn_S = absSktchIp_n(In_S);
    else
        ipn_S = dold_(:,In_S)'*data_n_;
        absIpn_S = abs(ipn_S);
    end    
    signIpn_S = sign(ipn_S);
        
    %----------------------------------------------------------------------
    % Compute residual and update dictionary
    %----------------------------------------------------------------------
    %residual = data_n_ - dold_(:,In_S) * (gramPar(In_S,In_S)\ipn_S) ;
    residual = data_n_ - dold_(:,In_S) * pinv( gramPar(In_S,In_S) ) * ipn_S ;
    
    dnewPart = real(residual*signIpn_S') + dold_(:,In_S)*diag(absIpn_S);
    dnewPart = sparse( repmat(I_d, 1, S)', ...
        reshape( repmat(In_S, 1, d)',[d*S 1] ), ...
        reshape( dnewPart, [1 d*S]), d, K );
    % dnew is now a reduction variable
    dnew = dnew + dnewPart;                
end

% Update data object internal counter
dataObj.n = dataObj.N;

end