classdef ictkm_audio_data < IcTKMData
% -------------------------------------------------------------------------
% IcTKM's audio training data type. 
% -------------------------------------------------------------------------
% Constructor input:
% -------------------------------------------------------------------------
%   - 'S': audio data sparsity level.
%   - 'audioFile': audio file name.
%   - 'numFramesPerBlock': deprecated input, should be set to 1.
%   - 'frameLengthInSeconds': length of audio training frame (block) in
%                             seconds.
%   - 'frameOffsetInSeconds': audio frame (block) offset duration in
%                             seconds.
%   - 'useSTFT': deprecated input, should be set to 0.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compressed Dictionary Learning Toolbox
% Copyright (C) Flavio Teixeira 2018
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, 
% Boston, MA  02110-1301, USA.    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    properties
        S                   = 0;
        ambient_d           = 0;
        intrinsic_d         = 0;
        signal              = [];
        signalLength        = 0;
        samplingFrequency   = 0;
        n                   = 0;
        N                   = 0;
        useGPU              = 0;
        signalIndex         = 0;
        numFramesPerBlock   = 0;
        frameLength         = 0;
        frameOffset         = 0;
        frameAmp            = 0;
        frame               = [];
        useSTFT             = 0;
        sizeFFT             = 0;      
        useWindowing        = 0;
    end
    methods
        %--- Constructor
        function obj = ictkm_audio_data(S, audioFile, numFramesPerBlock, ...
                frameLengthInSeconds, frameOffsetInSeconds, useSTFT)
            
            % load an audio file
            [obj.signal, obj.samplingFrequency] = audioread(audioFile);            
            
            obj.numFramesPerBlock = numFramesPerBlock;
            
            obj.ambient_d = ceil(obj.samplingFrequency*...
                frameLengthInSeconds*numFramesPerBlock);
            obj.intrinsic_d = obj.ambient_d;
            obj.S = S;
            
            % Sum up the channels to mono in case of stereo
            dim = size(obj.signal);
            if dim(2) > 1
                obj.signal = obj.signal(:, 1) + obj.signal(:, 2);
            end
            
            % represent x as column-vector
            obj.signal = obj.signal(:);
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % define analysis parameters
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            
            % length of the signal
            obj.signalLength = length(obj.signal);
            
            % frame length
            obj.frameLength = ...
                ceil(obj.samplingFrequency*frameLengthInSeconds);
            
            % hop size
            obj.frameOffset = ...
                ceil(obj.samplingFrequency*frameOffsetInSeconds);
            
            obj.useSTFT = useSTFT;
            
            useWindowing = 0;
            
            if (obj.useSTFT)
                % TODO: Revisit (check relationship between frame length,
                % DFT size, and the frequency resolution)
                % number of fft points
                obj.sizeFFT = 2*obj.frameLength;  % This gives a DFT frequency
                                                  % resolution of .5 Hz                                                                              
            end
            
            
            if useWindowing || useSTFT
                % form a periodic hamming window
                obj.frame = hamming(obj.frameLength, 'periodic');
                
                % coherent window amplification
                obj.frameAmp = sum(obj.frame)/obj.frameLength;
            else
                obj.frame = ones(obj.frameLength, 1);
                obj.frameAmp = 1;
            end                       
            
            % calculate the total number of columns
            obj.N = fix( (obj.signalLength-obj.frameLength)/...
                (obj.frameOffset*obj.numFramesPerBlock) );
            
            % initialize the signal segment time index
            obj.signalIndex = 0;
        end
        
        %--- Implement abstract method to get a data point
        function [obj, dataPoint] = GetDataPoint(obj)
            
            obj.n = obj.n + 1;
            if obj.n == obj.N+1
                obj.n = 0;
                obj.signalIndex = 0;
            end
            
            dataPointIdx = 0;
            dataPoint = zeros(obj.ambient_d, 1);
            for i=1:obj.numFramesPerBlock                                                          
                % windowing
                xw = obj.signal(obj.signalIndex+...
                    1:obj.signalIndex+obj.frameLength).*obj.frame;
                
                if (obj.useSTFT)
                    X = fft(xw, obj.sizeFFT);
                end
                
                % update the index
                obj.signalIndex = obj.signalIndex + obj.frameOffset;                
                %display([obj.signalIndex obj.signalLength]);                
                
                if (obj.useSTFT)
                    % Retrieve the power spectra
                    % take the amplitude of fft(x) and scale it, so not to be a
                    % function of the length of the window and its coherent
                    % amplification
                    dataPoint( (dataPointIdx+1):(dataPointIdx+obj.frameLength) ) = ...
                        X(1:obj.frameLength);
                    dataPoint( (dataPointIdx+1):(dataPointIdx+obj.frameLength) ) = ...
                        abs( dataPoint( (dataPointIdx+1):(dataPointIdx+obj.frameLength) ) )/...
                        obj.frameLength/obj.frameAmp;
                    
                    % correction of the DC & Nyquist component
                    if rem(obj.sizeFFT, 2)    % odd nfft excludes Nyquist point
                        dataPoint( (dataPointIdx+2):(dataPointIdx+obj.frameLength) ) = ...
                            dataPoint( (dataPointIdx+2):(dataPointIdx+obj.frameLength) ).*2;
                    else                      % even nfft includes Nyquist point
                        dataPoint( (dataPointIdx+2):(dataPointIdx+obj.frameLength-1) ) = ...
                            dataPoint( (dataPointIdx+2):(dataPointIdx+obj.frameLength-1) ).*2;
                    end
                else
                    dataPoint( (dataPointIdx+1):(dataPointIdx+obj.frameLength) ) = xw;
                end
                
                dataPointIdx = dataPointIdx + obj.frameLength - 1;
            end
        end
                
%         function delete(obj)
%         end
    end
end


