classdef ictkm_synthetic_data < ictkm_data
% -------------------------------------------------------------------------
% IcTKM's synthetic training data type. 
% -------------------------------------------------------------------------
% Constructor input:
% -------------------------------------------------------------------------
%   - 'ambient_d': ambient dimension (d)
%   - 'intrinsic_d': intrinsic dimension
%   - 'overcompletenessRatio': overcompleteness ratio (K/d)
%   - 'S': sparsity level
%   - 'T': deprecated input, should be set to S.
%   - 'N': number of training signals
%   - 'flatCoeff': set to 1 to use signals with flat sparse coefficients.
%   - 'dynamicRange': dynamic range of training signals.
%   - 'fixedCoeffDecay': set to 1 to use a fixed decay for the signal's 
%                        coefficients.
%   - 'rho': training signal's noise level.
%   - 'recThreshold': value for declaring dictionary learned from the 
%                     synthetic data as perfectly recovered.
%   - 'dicoType': Type of dictionary for generating the synthetic data.
%                 Following values are available:
%           - 'Random': uses a random generating dictionary.
%           - 'DiracDCT': uses the dxd Dirac basis plus the first 
%              (K-d) d-dimensional vectors of the discrete cosine basis.
%           - 'DiracHadamard': uses the dxd Dirac basis plus the first 
%              (K-d) d-dimensional vectors of the Hadamard basis.
%   - 'useGPU': set to 1 to speed-up vectorized calculations by using a 
%               CUDA GPU.
%    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compressed Dictionary Learning Toolbox
% Copyright (C) Flavio Teixeira 2018
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, 
% Boston, MA  02110-1301, USA.    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    properties
        dico                = [];
        dicoType            = [];
        S                   = 0;
        rho                 = 0;
        flatCoeff           = 0;
        fixedCoeffDecay     = 0;
        dynamicRange        = 0;
        T                   = 0;
        ambient_d           = 0;
        intrinsic_d         = 0;
        b                   = 0;
        K                   = 0;
        n                   = 0;
        N                   = 0;
        recThreshold        = 0;
        useGPU              = 0;
    end
    methods
        %--- Constructor
        function obj = ictkm_synthetic_data(ambient_d, intrinsic_d, ...
                overcompletenessRatio, S, T, N, flatCoeff, dynamicRange,...
                fixedCoeffDecay, rho, recThreshold, dicoType, useGPU)
            
            obj.S = S;
            obj.T = T;
            
            obj.N = N;           
            
            obj.useGPU = useGPU;                               
            
            obj.ambient_d = ambient_d;
            obj.intrinsic_d = intrinsic_d;
            obj.K = ceil(intrinsic_d*overcompletenessRatio);
            
            % -------------------------------------------------------------                        
            obj.flatCoeff = flatCoeff;            
            if (flatCoeff)
                obj.b = 0;
                obj.fixedCoeffDecay = 1;
            else                               
                % Find parameter b given the dynamic range
                obj.dynamicRange = dynamicRange;
                obj.fixedCoeffDecay = fixedCoeffDecay;  
                f = @(x) ( (1-x)^(1-obj.S) - obj.dynamicRange );
                % Wrong, below was assuming sparsity level of sqrt(d)
                %f = @(x) ( (1-x)^( 1-sqrt(obj.intrinsic_d) ) ...
                %    - obj.dynamicRange );
                a = 0;
                b = 1;
                n_iter = 50;
                [x, ~] = bisection_method(f, a, b, n_iter);
                obj.b = x;
            end            
            % -------------------------------------------------------------            
            obj.rho = rho;
            
            obj.dicoType = dicoType;
                        
            if strcmp(dicoType, 'Random')                               
                if obj.useGPU
                    sg = parallel.gpu.RandStream('CombRecursive',...
                        'NormalTransform', 'Inversion', 'Seed', 0);
                    parallel.gpu.RandStream.setGlobalStream(sg);
                    
                    obj.dico = randn(obj.intrinsic_d,obj.K, 'double', ...
                        'gpuArray');
                else
                    rng(0);
                    obj.dico = randn(obj.intrinsic_d,obj.K);
                end
                obj.dico = ...
                    obj.dico*diag( 1./sqrt( sum(obj.dico.*obj.dico) ) );
            elseif strcmp(dicoType, 'DiracDCT')
                if obj.useGPU
                    obj.dico = [eye(obj.intrinsic_d, 'double', ...
                        'gpuArray' ), dct( eye(obj.intrinsic_d, ...
                        'double', 'gpuArray') )];
                else
                    obj.dico = [eye(obj.intrinsic_d), ...
                        dct( eye(obj.intrinsic_d) )];
                end                
                obj.dico(:,(obj.K+1):2*obj.intrinsic_d) = [];
            elseif strcmp(dicoType, 'HaarDCT')
                %TODO: Implement this here
            elseif strcmp(dicoType, 'DiracHadamard')               
                obj.dico = [eye(obj.intrinsic_d), ...
                    hadamard(obj.intrinsic_d)/sqrt(obj.intrinsic_d)];
                obj.dico(:,(obj.K+1):2*obj.intrinsic_d)=[];
                if obj.useGPU
                    obj.dico = gpuArray(obj.dico, 'double');
                end
            end  
            if obj.useGPU
                obj.dico = [obj.dico; ...
                    zeros(obj.ambient_d-obj.intrinsic_d, obj.K, ...
                    'double', 'gpuArray')];
            else
                obj.dico = [obj.dico; ...
                    zeros(obj.ambient_d-obj.intrinsic_d, obj.K)];
            end            
            
            obj.recThreshold = recThreshold;                        
        end
        
        %--- Implement abstract method to get a data point
        function [obj, dataPoint] = GetDataPoint(obj)                     
            obj.n = obj.n + 1;
            if obj.n == obj.N+1
                obj.n = 0;
            end
            
            if obj.useGPU
                Svector = gpuArray.colon(1, obj.S);
                x1toT = zeros(obj.T, 1, 'double', 'gpuArray');
            else
                Svector = 1:obj.S;
                x1toT = zeros(obj.T, 1);
            end
            
            if obj.fixedCoeffDecay
                beta=1-obj.b;
            else
                beta=1-obj.b *rand(1, 1);
            end
            
            x1toS=sqrt(1./obj.S)*beta.^Svector';
            
            if obj.useGPU
                x1toSsign=2*round( rand(obj.S,1, 'double', ...
                    'gpuArray') ) - 1;
            else
                x1toSsign=2*round( rand(obj.S,1) ) - 1;
            end
            
            x1toS=x1toS.*x1toSsign;
            %norm(x1toS)
            if obj.T > obj.S
                if obj.useGPU
                    xSp1toT = randn(obj.T-obj.S, 1, 'double', 'gpuArray');
                else
                    xSp1toT = randn(obj.T-obj.S, 1);
                end
                
                xSp1toT=xSp1toT * sqrt(1-norm(x1toS)^2)/norm(xSp1toT);
                
                x1toT(1:obj.S) = x1toS;
                x1toT(obj.S+1:obj.T) = xSp1toT;
            else
                x1toT = x1toS/norm(x1toS);
            end
            
            if obj.useGPU
                p = gpuArray.randperm(obj.K);
            else
                p = randperm(obj.K);
            end
            
            sig = obj.dico( :,p(1:obj.T) ) * x1toT;
            if (obj.rho > 0)
                if obj.useGPU
                    noise = obj.rho*randn(obj.ambient_d, 1, ...
                        'double', 'gpuArray');
                else
                    noise = obj.rho*randn(obj.ambient_d, 1);
                end
                
                sig=(sig+noise)/sqrt(1+noise'*noise);
            end
            
            dataPoint = sig;
        end
        
               
        function delete(obj)
            if obj.useGPU
                gpuHandle = get_gpu_handle();
                clear obj.dico;
                wait(gpuHandle);
            end
        end
    end
end


