%% Compressed Dictionary Learning Toolbox
% The compressed dictionary learning toolbox provides functions to
% learn dictionaries from large-scale synthetic and audio training data with the
% Iterative Compressed-Thresholding and K-Residual-Means (IcTKM) algorithm
% introduced in [1]. In this documentation we 
% provide an overview of the toolbox and how to get started on a simple 
% dictionary learning example. We also discuss how the various toolbox
% scripts and functions are organized and how they can be used to learn
% dictionaries. Links to the audio files containing the sonification of 
% dictionaries learned on audio training data are provided.
% Lastly, we provide instructions for reproducing the numerical results 
% presented in [1].

%% Overview
% Sparsity in dictionary is an effective and widely used low-complexity
% model of high-dimensional data. In the sparse model we assume the
% existence of a dictionary in which every signal in the data class at hand
% can be approximated with a sparse linear expansion. Mathematically, we say that 
% there exists a set of $K$ unit-norm vectors $\phi_k \in \Re^d$ referred to as atoms,
% such that every signal $y \in \Re^d$ can be approximately represented in
% the dictionary as $y \approx \sum_{i
% \in I}x(i) \phi_i$, where $I$ is an index set and $x \in \Re^{K}$ is a sparse coefficient
% vector with $|I| = S$ and $S \ll d$. In dictionary learning we address
% the problem of (algorithmically) finding a dictionary which provides 
% sparse representations of a set of
% signals. In its most general form dictionary learning can be seen as a matrix
% factorization problem. Given a set of $N$ signals represented by the $d \times N$
% training data matrix $Y = (y_1, \ldots, y_N)$, decompose it into a $d \times K$
% dictionary matrix $\Phi = (\phi_1, \ldots, \phi_K)$ and a $K \times N$ coefficient matrix $X = (x_1, \ldots
% x_N)$, i.e., find $Y=\Phi X$ where every coefficient vector $x_k$ in $X$ is
% sparse. 
%
% The IcTKM algorithm is an alternating-minimization algorithm for 
% dictionary learning which alternates between 2 main steps: updating the sparse 
% coefficients based on the current version of the dictionary, and updating 
% the dictionary based on the current version of the coefficients. For sparsity 
% levels $S$ of practical interest, the most computationally expensive step 
% in the alternating minimization algorithm
% is the sparse coefficient update, which entails the calculation of the matrix 
% product between the training data $Y$ and the updated dictionary $\Psi$. 
% In the IcTKM algorithm, fast dictionary learning is achieved by compressing 
% $Y$ and $\Psi$ into $m \ll d$ dimensions with a Johnson-Lindenstrauss 
% randomized transformation $\Gamma$. More specifically, the matrix product $\Psi^{*}Y$
% is replaced by its compressed counterpart $\Psi^{*}\Gamma^{*}\Gamma Y$ in
% the sparse coefficient update step. When the Johnson-Lindestrauss transformation 
% is carried out with fast algorithms, the computational 
% complexity of the costly update is reduced by a factor of the order $O(m/d)$. 
% Given an input dictionary 
% $\Psi$, a training data matrix $Y$, and an embedding dimension $m$, each 
% iteration of the IcTKM algorithm is defined by the following steps:
%%
% # Draw the randomized Johnson-Lindenstrauss transform $\Gamma$.
% # For each training signals $y_n$ in the training set $Y$, find its sparse
% support $I^{ct}_{\Psi,n}$ via compressed thresholding: $I^{ct}_{\Psi,n} =
% \arg \max_{I:|I|=S} || \Psi_{I}^{*}\Gamma^{*}\Gamma y_n ||_1$.
% # For each atom $\bar{\psi}_k$ in the updated dictionary $\bar{\Psi}$,
% compute $K$-residual means: $\bar{\psi}_k = \frac{1}{N} \sum_n{
% sign(\langle \psi_k, y_n \rangle ) } \cdot 1_{I^{ct}_{\Psi,n}}(k) \cdot ( y_n - P(\Psi_{I^{ct}_{\Psi,n}}) y_n + P(\psi_k) y_n )$.
% # Normalize the updated atoms: $\bar{\Psi} = (
% \bar{\psi}_1/||\bar{\psi}_1||_2, \ldots, \bar{\psi}_K/|| \bar{\psi}_K ||_2 )$.
%
% In this toolbox we provide the ability to efficiently learn dictionaries 
% from high-dimensional audio and synthetic training data with IcTKM. 
% Efficiency is achieved by using randomized Johnson-Lindestrauss transforms 
% which can be carried out with the fast Fourier transform (FFT). Furthermore, 
% we provide the ability to fully utilize the available hardware with the
% toolbox. In particular, parallelization techniques can be used to process multiple 
% training signals with the available CPU cores. Furthermore, CUDA(R) capable 
% graphics processing units (GPUs) can be used to speed-up the most 
% computationally demanding vectorized calculations. 
% 
% The synthetic data is generated by using a sparse signal model
% with coefficients that follow a well-behaved distribution, i.e., the 
% non-zero values follow a geometric sequence which gracefully decays
% in magnitude. Mathematically, we let each coordinate of our coefficient 
% vector be defined as $x_{c,p,\sigma}(k) = \sigma(k) c(p(k))$, where $p$
% is a permutation sequence, $\sigma \in \{-1,1 \}^K$ is a sign sequence,
% and $c$ is a geometric sequence defined as $c(k)=c_b^k/||c||_2$ for $k \leq S$
% and $c(k)=0$ for $k > S$ with $c_b$ uniformly distributed in $[1-b,1]$ for
% some $0 < b < 1$. The synthetic signal then takes the form $y = (\Phi x_{c,p,\sigma} + r)/\sqrt{1 + ||r||_2^2}$,
% where $r \in \Re^d$ is noise defined by a Gaussian random vector with mean zero and
% variance $\rho^2$, see [1] for details. 
%
% The audio training data can be input 
% as an audio file in WAVE, FLAC, MP3, or OGG formats. The (potentially long) 
% signal in the input audio file is partitioned into smaller signals 
% which are allowed to overlap. Dictionary learning from audio data is 
% carried out directly from the time-domain audio samples with no frequency 
% transformation taking place, as opposed to what is usually done in common 
% audio tasks.

%% Getting Started
% In the following example code we will see how to use IctKM to learn dictionaries
% from synthetic training data. First we define the dictionary learning 
% problem parameters, i.e., the ambient dimension $d$, the embedding
% dimension $m$, and the number of atoms $K$ in the generating dictionary
% $\Phi$. In the example below we use $m=d/2$, which amounts to a
% data compression ratio of $2:1$, and an overcomplete generating
% dictionary with $K=(3/2)d$.
% 
%%
d = 256;                % signal dimension
m = ceil(d/2);          % embedding dimension
K = ceil(3*d/2);        % Number of atoms in the dictionary
%%
% Next, we define the training data generation parameters, i.e., the sparsity 
% level $S$, the Gaussian noise level $\rho$, the parameter $b$ for the
% geometric sequence, and the number of training signals $N$. We also set
% the random number generator seed for reproducible results.
%%
S = 2;                  % sparsity level (4,8,12,16)
rho = 0;                % noise level (0,1/sqrt(d))
b = 0.1;                % coefficient decay parameter (0,1)
N = 100000;             % number of training signals per iteration
rng(1);                 % random generator seed
%% 
% The generating dictionary $\Phi$ is defined as the union of two bases in
% $\Re^d$, the Dirac basis and the first half elements of the discrete
% cosine transform basis.
%%
dico = [eye(d),dct(eye(d))];
dico(:,K+1:2*d) = [];
%%
% To generate the synthetic training signals we invoke the toolbox function 
% <../../example/make_sparse_signal.m make_sparse_signal.m>
%%
Y = make_sparse_signal(dico,N,S,S,b,rho);
%%
% The training signals are now stored in the data matrix $Y$.
% We now set the initialization dictionary $\Psi$ uniformly at random from the
% unit sphere in $\Re^d$ and run $100$ iterations of the IcTKM algorithm 
% with the toolbox function <../../example/ictkm_simplified.m ictkm_simplified.m>
%% 
dinit = randn(d,K);                                     % Create input dictionary
dinit = dinit*diag(1./sqrt(sum(dinit.*dinit)));         % Normalize input dictionary
maxit = 100;                                            % Set maximum number of iterations
rdico = ictkm_simplified(Y, K, S, m, maxit, dinit);    % Run the IcTKM algorithm
%%
% The output (recovered) dictionary $\bar{\Psi}$ is now stored in the matrix rdico. 
% We now evaluate the recovery performance of IcTKM. Given an atom
% $\bar{\psi}_l$ from $\bar{\Psi}$, the criteria for
% declaring the generating atom $\phi_k$ as recovered is obtained by
% computing the dictionary correlation inner product $\langle \bar{\psi}_l, \phi_k
% \rangle$, more specifically via $\max_l | \langle \bar{\psi}_l, \phi_k \rangle | \geq 0.99$. 
% In the example code below we count the number of recovered atoms following 
% this criteria and compute the percentage of recovered atoms.
%%
rcorr=max(abs(rdico'*dico));
rec_count = find(rcorr > 0.99);                 % Count the number of recovered atoms
percent_recov = (length(rec_count)/K)*100;      % Calculate percentage of recovered atoms
display(['percent of recovered atoms: ' num2str(percent_recov) '%']);
%%
% To run this dictionary learning example and for more details, refer to 
% the toolbox script <../../example/ictkm_example_script.m ictkm_example_script.m>

%% Listening to the Audio Dictionaries
% For the audio training data we have used several audio recordings from the 
% <https://staff.aist.go.jp/m.goto/RWC-MDB/ RWC database>. 
% The recordings were comprised of stereo audio
% signals sampled at 44.1 KHz. We have first summed the audio
% signals to mono and then partioned the resulting signals into smaller
% blocks of $0.25$ and $1$ second durations. The blocks were allowed to
% overlap such that the maximally amount of overlap of one block with a
% shifted version of itself varied from $95\%$ for the $0.25$ seconds block,
% up to $98.75\%$ for the $1$ second block, see Section 4.2 of [1] for more
% details. We ran $200$ iterations of IcTKM using a compression ratio of $5:1$
% to learn the dictionaries. The sparsity level was fixed at $S=4$ and we have
% learned dictionaries of size $K=64$ and $K=256$ atoms. To create an audio 
% representation of the learned dictionaries, we have first sorted the atoms 
% by their fundamental frequency, normalized their entries in the range $(-1,1)$, 
% and then concatenated the renormalized atoms in a long array. 
% A pause of $0.5$ seconds has also been added between each atom for an 
% improved listening experience. The data in the long array 
% was saved to audio files, which can be listened by following the links
% below:
%%
% * classical piano recordings - 0.25 seconds training blocks:
% <../../simulation/results/audio_files/IcTKM_audio_simulation_audioFile_classical_piano.wav_K_64_S_4_frameLeng_0.25_frameOff_0.0125_compressRatio_5_numFramesPerBlock_1_FJLTType_PartialDCT_embedding_numIter_200_03_02_2018_14_37_08.ogg
% K=64> and
% <../../simulation/results/audio_files/IcTKM_audio_simulation_audioFile_classical_piano.wav_K_256_S_4_frameLeng_0.25_frameOff_0.0125_compressRatio_5_numFramesPerBlock_1_FJLTType_PartialDCT_embedding_numIter_200_03_02_2018_15_05_24.ogg
% K=256> and 1 second training blocks: <../../simulation/results/audio_files/IcTKM_audio_simulation_audioFile_classical_piano.wav_K_64_S_4_frameLeng_1_frameOff_0.0125_compressRatio_5_numFramesPerBlock_1_FJLTType_PartialDCT_embedding_numIter_200_03_03_2018_05_13_41.ogg
% K=64> and
% <../../simulation/results/audio_files/IcTKM_audio_simulation_audioFile_classical_piano.wav_K_256_S_4_frameLeng_1_frameOff_0.0125_compressRatio_5_numFramesPerBlock_1_FJLTType_PartialDCT_embedding_numIter_200_03_03_2018_11_08_20.ogg
% K=256>
% * flamenco recordings - 0.25 seconds training blocks:
% <../../simulation/results/audio_files/IcTKM_audio_simulation_audioFile_flamenco.wav_K_64_S_4_frameLeng_0.25_frameOff_0.0125_compressRatio_5_numFramesPerBlock_1_FJLTType_PartialDCT_embedding_numIter_200_03_02_2018_15_30_54.ogg
% K=64> and
% <../../simulation/results/audio_files/IcTKM_audio_simulation_audioFile_flamenco.wav_K_256_S_4_frameLeng_0.25_frameOff_0.0125_compressRatio_5_numFramesPerBlock_1_FJLTType_PartialDCT_embedding_numIter_200_03_02_2018_16_01_59.ogg
% K=256> and 1 second training blocks: <../../simulation/results/audio_files/IcTKM_audio_simulation_audioFile_flamenco.wav_K_64_S_4_frameLeng_1_frameOff_0.0125_compressRatio_5_numFramesPerBlock_1_FJLTType_PartialDCT_embedding_numIter_200_03_03_2018_09_37_59.ogg
% K=64> and
% <../../simulation/results/audio_files/IcTKM_audio_simulation_audioFile_flamenco.wav_K_256_S_4_frameLeng_1_frameOff_0.0125_compressRatio_5_numFramesPerBlock_1_FJLTType_PartialDCT_embedding_numIter_200_03_03_2018_13_31_56.ogg
% K=256>
% * folk japanese recordings - 0.25 seconds training blocks:
% <../../simulation/results/audio_files/IcTKM_audio_simulation_audioFile_folk_japanese.wav_K_64_S_4_frameLeng_0.25_frameOff_0.0125_compressRatio_5_numFramesPerBlock_1_FJLTType_PartialDCT_embedding_numIter_200_03_02_2018_15_18_30.ogg
% K=64> and
% <../../simulation/results/audio_files/IcTKM_audio_simulation_audioFile_folk_japanese.wav_K_256_S_4_frameLeng_0.25_frameOff_0.0125_compressRatio_5_numFramesPerBlock_1_FJLTType_PartialDCT_embedding_numIter_200_03_02_2018_14_26_32.ogg
% K=256> and 1 second training blocks: <../../simulation/results/audio_files/IcTKM_audio_simulation_audioFile_folk_japanese.wav_K_64_S_4_frameLeng_1_frameOff_0.0125_compressRatio_5_numFramesPerBlock_1_FJLTType_PartialDCT_embedding_numIter_200_03_03_2018_02_37_08.ogg
% K=64> and
% <../../simulation/results/audio_files/IcTKM_audio_simulation_audioFile_folk_japanese.wav_K_256_S_4_frameLeng_1_frameOff_0.0125_compressRatio_5_numFramesPerBlock_1_FJLTType_PartialDCT_embedding_numIter_200_03_03_2018_07_42_43.ogg
% K=256>

%% Organization of the Toolbox
% The compressed dictionary learning toolbox contains several
% folders, which have been organized with functions and scripts of related purpose. 
% We now list these folders and describe their most important functions. 
% For more details refer to the toolbox m-files, which can be accessed by 
% following the links below.
%% 
% * <../../algorithm/ algorithm>: This directory contains the IcTKM
% algorithm and auxiliary scripts. 
% In <../../algorithm/ictkm_redraw_embedding.m ictkm_redraw_embedding.m>
% we have the function to construct randomized Johnson-Lindenstrauss
% embeddings based on the Discrete Cosine Transform (DCT), Discrete Fourier 
% Transform (DFT), and the Circulant Rademacher Transform (CRT). In 
% <../../algorithm/ictkm_core.m ictkm_core.m> and 
% <../../algorithm/ictkm_core_parfor.m ictkm_core_parfor.m>
% we have the functions to update the coefficients and dictionary, sequentially
% or in parallel by using the parfor construction. Lastly, 
% in <../../algorithm/ictkm.m ictkm.m> we have the IcTKM algorithm
% implementation with the main algorithm flow.
% * <../../data/ data>: This directory contains the MATLAB classes for
% generating synthetic training signals <../../data/ictkm_synthetic_data.m ictkm_synthetic_data.m>
% and audio training signals <../../data/ictkm_audio_data.m
% ictkm_audio_data.m>. These two classes inherit from the abstract data 
% class <../../data/ictkm_data.m ictkm_data.m> which contains the interface
% getDataPoint() for online data generation. With this
% abstract class interface we can remove the dependence of the IcTKM algorithm 
% on a particular training data type.
% * <../../example/ example>: Contains the scripts to reproduce the dictionary 
% learning example in the Getting Started section of this documentation.
% * <../../init/ init>: Contains functions for creating an input
% dictionary for the IcTKM algorithm, such as in <../../init/ictkm_dico_init.m
% ictkm_dico_init.m>, as well as miscellaneous initialization functions used 
% in the toolbox.
% * <../../simulation/ simulation>: Contains the scripts for reproducing the
% numerical simulations presented in [1]. In particular, 
% <../../simulation/ictkm_synthetic_simulation.m ictkm_synthetic_simulation.m>
% and <../../simulation/ictkm_scalability_simulation.m ictkm_scalability_simulation.m>
% can be used for running the dictionary learning simulations with synthetic data in 
% Secs. 4.1.1 to 4.1.3 and the scalability simulations of Section
% 4.1.4, respectively. Similarly, <../../simulation/ictkm_audio_simulation.m ictkm_audio_simulation.m>
% has been used to run the audio simulations of Sec. 4.2. 
% Lastly, <../../simulation/ictkm_simulation.m ictkm_simulation.m>
% is the main simulation function which is used by the specific simulation 
% scripts in the folder.
% * <../../utils/ utils>: Contains utilities scripts, such as function for
% synthesizing the learned dictionaries into audio files, console
% formatting functions, a fast DCT implementation, and utilities functions
% for properly initializing and using GPU acceleration.
%
%%
% 
% 
% In the following figure we illustrate the dependencies between the
% most important functions in the toolbox. 
%%
% 
% <<../latex/tikz/toolbox_organization-figure0.jpg>>
% 
%% Reproducing the Numerical Results
% As discussed in the previous section, the toolbox folder <../../simulation/ simulation>
% contains all the required scripts for reproducing the numerical
% simulations of Section 4 in [1]. Specifically, the results for the synthetic 
% training data in Sections 4.1.1 to 4.1.3 can be reproduced by using 
% <../../simulation/ictkm_synthetic_simulation.m ictkm_synthetic_simulation.m>.
% The scalability experiments of Section 4.1.4 can be reproduced by using 
% <../../simulation/ictkm_scalability_simulation.m ictkm_scalability_simulation.m>.
% Lastly, the audio experiments of Sections 4.2.1 and 4.2.2 can be reproduced
% by using <../../simulation/ictkm_audio_simulation.m ictkm_audio_simulation.m>.
% For complete reference, we now describe in details the options structure 
% used by the main simulation script <../../simulation/ictkm_simulation.m ictkm_simulation.m>:
%%
% results = ictkm_simulation(options);
%%
% The IcTKM algorithm can be configured with the option structure fields in the 
% following table. Here we can set the number of iterations, whether the
% dictionary correlation inner products $\langle \bar{\psi}_l, \phi_k \rangle$
% are stored in the results structure for further inspection, and if the 
% algorithm displays its status in the command window.
%%
% <html>
% <table border=1>
% <caption>IcTKM Algorithm Configuration</caption>
% <tr>
%   <th>Field</th>
%   <th>Description</th>
% </tr>
% <tr>
%   <td>options.IcTKM_numIter</td>
%   <td>Sets the number of iterations</td>
% </tr>
% <tr>
%   <td>options.IcTKM_storeRecCorr</td>
%   <td>Set to 1 to return the dictionary correlation inner products</td>
% </tr>
% <tr>
%   <td>options.IcTKM_verbose</td>
%   <td>Set to 1 to display the algorithm progress and status in the console</td>
% </tr>
% </table>
% </html>
%
% The run mode of the IcTKM algorithm can be configured with the
% struct fields described in the next table. The algorithm can be
% configured to run in parallel mode where the dictionary update loop is
% parallelized via 'parfor'. The algorithm can also be set to run in CUDA(R) 
% enabled GPUs such that all vectorized computations will be performed
% directly in the GPU.
%
%%
% <html>
% <table border=1>
% <caption>IcTKM Run Mode Configuration</caption>
% <tr>
%   <th>Field</th>
%   <th>Description</th>
% </tr>
% <tr>
%   <td>options.IcTKM_runMode_parallel</td>
%   <td>Set to 1 to enable parallelization of dictionary updates</td>
% </tr>
% <tr>
%   <td>options.IcTKM_runMode_parallel_boost_workers</td>
%   <td>Set to 1 to use the maximum number of available threads</td>
% </tr>
% <tr>
%   <td>options.IcTKM_runMode_useGPU</td>
%   <td>Set to 1 to use a CUDA capable GPU to speed-up vectorized calculations</td>
% </tr>
% </table>
% </html>
%
% The configuration details of the Johnson-Lindenstrauss (JL) embedding are 
% described in the following table. Here we can use JL embeddings based on 
% orthogonal transforms such as the unitary discrete Fourier and cosine 
% matrices, and also circulant matrix constructions. The algorithm can also
% be set to run with no data embedding.
%
%%
% <html>
% <table border=1>
% <caption>Johnson-Lindenstrauss Embedding Configuration</caption>
% <tr>
%   <th>Field</th>
%   <th>Description</th>
% </tr>
% <tr>
%   <td>options.IcTKM_embedding_PartialFourier</td>
%   <td>Set to 1 to use a JL matrix based on the partial discrete Fourier construction</td>
% </tr>
% <tr>
%   <td>options.IcTKM_embedding_PartialDCT</td>
%   <td>Set to 1 to use a JL matrix based on the partial discrete cosine construction. </td>
% </tr>
% <tr>
%   <td>options.IcTKM_embedding_PartialCirculant</td>
%   <td>Set to 1 to use a JL matrix based on the circulant construction</td>
% </tr>
% <tr>
%   <td>options.IcTKM_embedding_dimension</td>
%   <td>Sets the dimension of the embedded space</td>
% </tr>
% <tr>
%   <td>options.IcTKM_embedding_disabled</td>
%   <td>Set to 1 to disable data embedding</td>
% </tr>
% </table>
% </html>
%
% The dictionary learning configuration parameters are described next. In
% particular, here we can set the ambient dimension $d$, the dictionary
% overcomplete ratio $K/d$, the sample size ratio $\alpha$, where 
% $N= \alpha \cdot K \log K$, and the toolbox random number generator seed
% for reproducible results.
%
%%
% <html>
% <table border=1>
% <caption>Simulation Configuration</caption>
% <tr>
%   <th>Field</th>
%   <th>Description</th>
% </tr>
% <tr>
%   <td>options.ambientDimension</td>
%   <td>Sets the learning problem ambient dimension</td>
% </tr>
% <tr>
%   <td>options.overcompletenessRatio</td>
%   <td>Sets the learning problem overcompleteness ratio</td>
% </tr>
% <tr>
%   <td>options.SampleSizeRatio</td>
%   <td>Sets the learning problem sample size ratio</td>
% </tr>
% <tr>
%   <td>options.randomSeed</td>
%   <td>Sets the random seed for reproducible results</td>
% </tr>
% </table>
% </html>
%
% The struct fields for configuring the synthetic training
% data generation are described next. Here we can set the training data
% sparsity level $S$, the noise level $\rho$, and whether a 'flat' generating
% sequence $c$ is used where $c(k)=1/\sqrt{S}$ for $k \leq S$ and $c(k) = 0$
% for $k > S$, or a geometric sequence $c$ is used where $c(k)=c_b^k/||c||_2$ 
% for $k \leq S$ and $c(k)=0$ for $k > S$ with $c_b$ uniformly distributed in 
% $[1-b,1]$ for some $0 < b < 1$. Here we can also set the training data
% dynamic range and the signal-to-noise ratio 
% $E( || \Phi x_{c,p,\sigma} ||_2^2) / E( ||r ||_2^2 )$. Lastly, we may
% also set the type of generating dictionary $\Phi$; uniformly at random from the
% unit sphere in $\Re^d$ and as the union of two bases in
% $\Re^d$, i.e., the Dirac basis and the first half elements of the discrete
% cosine (or Hadamard) transform basis.
%
%%
% <html>
% <table border=1>
% <caption>Synthetic Training Data Configuration</caption>
% <tr>
%   <th>Field</th>
%   <th>Description</th>
% </tr>
% <tr>
%   <td>options.DataModeSynthetic</td>
%   <td>Set to 1 to use synthetic training data in the simulation</td>
% </tr>
% <tr>
%   <td>options.SyntheticData_S</td>
%   <td>Sets the synthetic data sparsity level</td>
% </tr>
% <tr>
%   <td>options.SyntheticData_noiseLevel</td>
%   <td>Sets the synthetic data noise level</td>
% </tr>
% <tr>
%   <td>options.SyntheticData_flatCoeff</td>
%   <td>Set to 1 to use data with flat coefficients</td>
% </tr>
% <tr>
%   <td>options.SyntheticData_fixedCoeffDecay</td>
%   <td>Set to 1 to use data with a fixed coefficient decay</td>
% </tr>
% <tr>
%   <td>options.SyntheticData_dynamicRange</td>
%   <td>Sets the synthetic data dynamic range in dB</td>
% </tr>
% <tr>
%   <td>options.SyntheticData_origDicoRandom</td>
%   <td>Set to 1 to use the random dictionary for generating the synthetic data</td>
% </tr>
% <tr>
%   <td>options.SyntheticData_origDicoDiracDCT</td>
%   <td>Set to 1 to use the Dirac+DCT dictionary for generating the synthetic data</td>
% </tr>
% <tr>
%   <td>options.SyntheticData_origDicoDiracHadamard</td>
%   <td>Set to 1 to use the Dirac+Hadamard dictionary for generating the synthetic data</td>
% </tr>
% </table>
% </html>
%
% The the audio training data configuration is described in the next 
% table. Here we can set the training data sparsity
% level $S$ and the audio file path. As we partition the audio file in smaller
% blocks (frames), we can also set the size of the audio frames and the
% amount of overlap of one frame to the next.
%
%%
% <html>
% <table border=1>
% <caption>Audio Training Data Configuration</caption>
% <tr>
%   <th>Field</th>
%   <th>Description</th>
% </tr>
% <tr>
%   <td>options.DataModeAudio</td>
%   <td>Set to 1 to use audio training data in the simulation</td>
% </tr>
% <tr>
%   <td>options.AudioData_S</td>
%   <td>Set the audio data sparsity level</td>
% </tr>
% <tr>
%   <td>options.AudioData_frameLengthInSeconds</td>
%   <td>Set the audio data frame length in seconds</td>
% </tr>
% <tr>
%   <td>options.AudioData_frameOffsetInSeconds</td>
%   <td>Set the frame offset duration in seconds</td>
% </tr>
% <tr>
%   <td>options.AudioData_fileName</td>
%   <td>Set the location of the audio training file</td>
% </tr>
% </table>
% </html>
%
% Lastly, in the following table we describe the struct fields for
% configuring the dictionary initialization for the IcTKM algorithm.
%
%%
% <html>
% <table border=1>
% <caption>Dictionary Initialization Configuration</caption>
% <tr>
%   <th>Field</th>
%   <th>Description</th>
% </tr>
% <tr>
%   <td>options.DicoInit_random</td>
%   <td>Set to 1 to use a random input dictionary</td>
% </tr>
% <tr>
%   <td>options.DicoInit_oracle</td>
%   <td>Set to 1 to use the generating dictionary as the input dictionary</td>
% </tr>
% </table>
% </html>
% 
%% References
% <html>
%   <div style="width: 100%; border: 0 solid white;">
%       <div style="float:left;">[1]</div>
%       <div style="padding: 0px 0px 0px 5px; overflow: hidden;">
%           F. Teixeira and K. Schnass, "Compressed Dictionary Learning,"
%           <i>arXiv 1805.00692</i>, May, 2018. 
%       </div>     
%   </div>
% </html>