function ictkm_driver_compute_cluster_audio_simulation(...
                                                        audioFileIndex, ...
                                                        numberAudioFeatures, ...
                                                        sparsityLevel, ...
                                                        numFramesPerBlock,...
                                                        frameDuration,...
                                                        frameHopDuration,...
                                                        compressionRatio,...
                                                        numIterations,...
                                                        useScratchSpace...
                                                        )
% -------------------------------------------------------------------------
% Driver function to start an audio simulation on University of 
% Innsbruck's compute cluster.
% -------------------------------------------------------------------------
% Compile this as follows to run on the cluster:
% =========================================================================
% mcc -mv -R -nojvm -R -singleCompThread ictkm_driver_compute_cluster_audio_simulation.m
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compressed Dictionary Learning Toolbox
% Copyright (C) Flavio Teixeira 2018
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor,
% Boston, MA  02110-1301, USA.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% =========================================================================
% Deployed application have to treat input parameters differently
% =========================================================================
if isdeployed
    audioFileIndex = str2double(audioFileIndex);
    numberAudioFeatures = str2double(numberAudioFeatures);
    sparsityLevel = str2double(sparsityLevel);
    numFramesPerBlock = str2double(numFramesPerBlock);
    frameDuration = str2double(frameDuration);
    frameHopDuration = str2double(frameHopDuration);
    numIterations = str2double(numIterations);
    compressionRatio = str2double(compressionRatio);
    useScratchSpace = str2double(useScratchSpace);
end

% =========================================================================
% Set global random seed for reproducible results
% =========================================================================
randomSeed = 1;

% =========================================================================
% Set fast Johnson-Lindenstrauss transform
% =========================================================================
IcTKM_embedding_PartialFourier = 0;
IcTKM_embedding_PartialDCT = 1;
IcTKM_embedding_PartialCirculant = 0;

% =========================================================================
% learn audio directly from the time-domain samples
% =========================================================================
useSTFT = 0;

% =========================================================================
% Create audio filename
% =========================================================================
path_string = 'training_data/genres/';
filename_array = ...
    [{'classical_brass_band.wav'}, {'classical_orchestral.wav'},...
    {'classical_piano.wav'}, {'flamenco.wav'}, {'funk.wav'},...
    {'heavy_metal.wav'}, {'jazz.wav'}, {'techno.wav'} ...
    {'folk_japanese.wav'}];
audioFile = strcat(path_string, filename_array{audioFileIndex});

if useScratchSpace
    audioFile = strcat('/scratch/c7021060/', audioFile);
end

% =========================================================================
% Generate filename based on simulation parameters
% =========================================================================

FJLT_type_str = []; %#ok<NASGU>
if compressionRatio ~= 1
    if IcTKM_embedding_PartialFourier
        FJLT_type_str = 'PartialFourier_embedding'; %#ok<UNRCH>
    elseif IcTKM_embedding_PartialDCT 
        FJLT_type_str = 'PartialDCT_embedding';
    elseif IcTKM_embedding_PartialCirculant %#ok<UNRCH>
        FJLT_type_str = 'PartialCirculant_embedding';
    else
        FJLT_type_str = [];
    end
else
    FJLT_type_str = 'no_embedding';
end

formatOut = 'mm_dd_yyyy_HH_MM_SS';
date_str = datestr(now,formatOut);

audioFileShort = strsplit(audioFile,'/');

resultsFilename = ...
    strsplit(mfilename('fullpath'),['compute_cluster/' mfilename]);
resultsFilename = resultsFilename{1};

resultsFilename = ...
    [resultsFilename '/results/IcTKM_audio_simulation_audioFile_' ...
    audioFileShort{end} ...
    '_K_' num2str(numberAudioFeatures) ...
    '_S_' num2str(sparsityLevel) ...
    '_frameLeng_' num2str(frameDuration) ...
    '_frameOff_' num2str(frameHopDuration) ...
    '_compressRatio_' num2str(compressionRatio) ...
    '_numFramesPerBlock_' num2str(numFramesPerBlock) ...
    '_FJLTType_' num2str(FJLT_type_str) ...
    '_numIter_' num2str(numIterations) '_' date_str];

if useScratchSpace
    resultsFilename = strcat('/scratch/c7021060/', resultsFilename);
end

verboseAlgorithm = 1;

% This is not supported in the compute cluster
useParallelization = 0;
useGpuAcceleration = 0;

% =========================================================================
% Run the simulation
% =========================================================================
ictkm_audio_simulation(...
    randomSeed,...
    numberAudioFeatures,...
    sparsityLevel,...
    useSTFT, ...
    audioFile, ...
    numFramesPerBlock, ...
    frameDuration,...
    frameHopDuration,...
    compressionRatio, ...
    numIterations,...
    IcTKM_embedding_PartialFourier, ...
    IcTKM_embedding_PartialDCT, ...
    IcTKM_embedding_PartialCirculant,...
    verboseAlgorithm,...
    useParallelization,...
    useGpuAcceleration,...
    resultsFilename);
