function ictkm_audio_simulation(...
                                    randomSeed,...
                                    numberAudioFeatures,...
                                    sparsityLevel,...
                                    useSTFT, ...
                                    audioFile, ...
                                    numFramesPerBlock, ...
                                    frameLengthInSeconds,...
                                    frameOffsetInSeconds,...
                                    IcTKM_compresssionRatio, ...
                                    IcTKM_numIterations,...
                                    IcTKM_embedding_PartialFourier, ...
                                    IcTKM_embedding_PartialDCT, ...
                                    IcTKM_embedding_PartialCirculant,...
                                    verboseAlgorithm,...
                                    useParallelization,...
                                    useGpuAcceleration,...
                                    resultsFilename...
                                )
% -------------------------------------------------------------------------
% Runs a dictionary learning audio simulation with IcTKM.
% -------------------------------------------------------------------------
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compressed Dictionary Learning Toolbox
% Copyright (C) Flavio Teixeira 2018
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, 
% Boston, MA  02110-1301, USA.    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% =========================================================================
% Display parameters
% =========================================================================
disp('ictkm_audio_simulation parameters');
disp([sprintf('\t') 'audioFile: ' audioFile ]);
disp([sprintf('\t') 'sparsityLevel: ' ...
    num2str(sparsityLevel) ]);
disp([sprintf('\t') 'numberAudioFeatures: ' ...
    num2str(numberAudioFeatures) ]);
disp([sprintf('\t') 'frameLengthInSeconds: ' ...
    num2str(frameLengthInSeconds) ]);
disp([sprintf('\t') 'frameOffsetInSeconds: ' ...
    num2str(frameOffsetInSeconds) ]);
disp([sprintf('\t') 'IcTKM_compresssionRatio: ' ...
    num2str(IcTKM_compresssionRatio) ]);
disp([sprintf('\t') 'IcTKM_numIterations: ' ...
    num2str(IcTKM_numIterations) ]);
disp([sprintf('\t') 'IcTKM_embedding_PartialFourier: ' ...
    num2str(IcTKM_embedding_PartialFourier) ]);
disp([sprintf('\t') 'IcTKM_embedding_PartialDCT: ' ...
    num2str(IcTKM_embedding_PartialDCT) ]);
disp([sprintf('\t') 'IcTKM_embedding_PartialCirculant: ' ...
    num2str(IcTKM_embedding_PartialCirculant) ]);


optsIcTKMSimulation=[];

% =========================================================================
% Set IcTKM misc configurations
% =========================================================================
optsIcTKMSimulation.IcTKM_numIter = IcTKM_numIterations;
optsIcTKMSimulation.IcTKM_verbose = verboseAlgorithm;
optsIcTKMSimulation.IcTKMHaltOnPerfectRecovery = 0;
optsIcTKMSimulation.randomSeed = randomSeed;

if useSTFT
    optsIcTKMSimulation.IcTKM_nonNegativeConstraint = 1;
else
    optsIcTKMSimulation.IcTKM_nonNegativeConstraint = 0;
end

% =========================================================================
% Sets the IcTKM Run mode
% =========================================================================
optsIcTKMSimulation.IcTKM_runMode_parallel = useParallelization;
optsIcTKMSimulation.IcTKM_runMode_parallel_boost_workers = 0;
optsIcTKMSimulation.IcTKM_runMode_useGPU = useGpuAcceleration;

% =========================================================================
% Create initialization for the IcTKM
% =========================================================================
optsIcTKMSimulation.DicoInit_random = 1;

% =========================================================================
% Define the problem size
% =========================================================================
[~,audioSamplingRate] = audioread(audioFile); % Retrieve audio sampling rate

optsIcTKMSimulation.ambientDimension = ...
    ceil(frameLengthInSeconds*audioSamplingRate*numFramesPerBlock);
optsIcTKMSimulation.intrinsicDimension = ...
    optsIcTKMSimulation.ambientDimension;

% We will usually have an undercomplete representation for audio,
% e.g., music transcription where K is the number of keys in a musical
% piece.
optsIcTKMSimulation.overcompletenessRatio = ...
    numberAudioFeatures/optsIcTKMSimulation.ambientDimension;

% =========================================================================
% Create data set for learning the dictionary
% =========================================================================
optsIcTKMSimulation.DataModeAudio = 1;

% S maxes out to sqrt(d) for arbitrary target errors.
optsIcTKMSimulation.AudioData_S = sparsityLevel;

optsIcTKMSimulation.AudioData_numFramesPerBlock = numFramesPerBlock;
optsIcTKMSimulation.AudioData_frameLengthInSeconds = frameLengthInSeconds;
optsIcTKMSimulation.AudioData_frameOffsetInSeconds = frameOffsetInSeconds;

optsIcTKMSimulation.AudioData_fileName = audioFile;

% =========================================================================
% Configure the Johnson-Lindenstrauss embedding
% =========================================================================
if (IcTKM_compresssionRatio == 1)
    optsIcTKMSimulation.IcTKM_embedding_disabled = 1;
else
    optsIcTKMSimulation.IcTKM_embedding_disabled = 0;
    optsIcTKMSimulation.IcTKM_embedding_PartialFourier = ...
        IcTKM_embedding_PartialFourier;
    optsIcTKMSimulation.IcTKM_embedding_PartialDCT = ...
        IcTKM_embedding_PartialDCT;
    optsIcTKMSimulation.IcTKM_embedding_PartialCirculant = ...
        IcTKM_embedding_PartialCirculant;
    optsIcTKMSimulation.IcTKM_embedding_dimension = ...
        ceil(optsIcTKMSimulation.ambientDimension/IcTKM_compresssionRatio);
end

% =========================================================================
% Run the IcTKM simulation
% =========================================================================
results = ictkm_simulation(optsIcTKMSimulation);

% =========================================================================
% Collect results
% =========================================================================
dico = results.rdico; 
dataObj = results.dataObj;

% Save dictionary to matlab file
save([resultsFilename '.mat'] , 'dico');

% Save synthesized dictionary to audio file
dico_audio = ictkm_synthesize_dictionary_to_audio(dataObj, dico);

audiowrite([resultsFilename '.ogg'], dico_audio, ...
    dataObj.samplingFrequency);

