function results = ictkm_simulation(options)
% -------------------------------------------------------------------------
% Runs a dictionary learning simulation with IcTKM. Refer to the toolbox
% documentation for more details and usage instructions.
% -------------------------------------------------------------------------
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compressed Dictionary Learning Toolbox
% Copyright (C) Flavio Teixeira 2018
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, 
% Boston, MA  02110-1301, USA.    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% =========================================================================
% options structure
% =========================================================================
%opts.IcTKM_numIter
%opts.IcTKM_verbose
%opts.IcTKM_nonNegativeConstraint
% -------------------------------------------------------------------------
%opts.ambientDimension
%opts.intrinsicDimension
%opts.overcompletenessRatio
%opts.SampleSizeRatio
%opts.randomSeed
% -------------------------------------------------------------------------
%opts.IcTKM_embedding_PartialFourier
%opts.IcTKM_embedding_PartialDCT
%opts.IcTKM_embedding_PartialCirculant

%opts.IcTKM_embedding_dimension

%opts.IcTKM_embedding_disabled
% -------------------------------------------------------------------------
%opts.IcTKM_runMode_parallel
%opts.IcTKM_runMode_parallel_boost_workers

%opts.IcTKM_runMode_sequential

%opts.IcTKM_runMode_useGPU
% -------------------------------------------------------------------------
%opts.DataModeSynthetic
%opts.SyntheticData_S
%opts.SyntheticData_noiseLevel
%opts.SyntheticData_flatCoeff
%opts.SyntheticData_fixedCoeffDecay
%opts.SyntheticData_dynamicRange
%opts.SyntheticData_recThres
%opts.SyntheticData_origDicoRandom
%opts.SyntheticData_origDicoDiracDCT
%opts.SyntheticData_origDicoHaarDCT
%opts.SyntheticData_origDicoDiracHadamard
% -------------------------------------------------------------------------
%opts.DataModeAudio
%opts.AudioData_S
%opts.AudioData_frameLengthInSeconds
%opts.AudioData_frameOffsetInSeconds
%opts.AudioData_numFramesPerBlock
%opts.AudioData_fileName
% -------------------------------------------------------------------------
%opts.DataModeImagePatches
% -------------------------------------------------------------------------
%opts.DicoInit_random
%opts.DicoInit_oracle
%opts.DicoInit_1_1_ratio
%opts.DicoInit_1_4_ratio
% =========================================================================
options = validate_options(options);
% -------------------------------------------------------------------------
% Set path
% -------------------------------------------------------------------------
if ~isdeployed
    ictkm_set_path();
end
% -------------------------------------------------------------------------
% Set options for IcTKM algorithm
% -------------------------------------------------------------------------
optsIcTKM=[];
% -------------------------------------------------------------------------
optsIcTKM.numIter = options.IcTKM_numIter;
% -------------------------------------------------------------------------
optsIcTKM.verbose = options.IcTKM_verbose;
% -------------------------------------------------------------------------
optsIcTKM.nonNegative = options.IcTKM_nonNegativeConstraint;
% -------------------------------------------------------------------------
optsIcTKM.d = options.ambientDimension;
optsIcTKM.K = ceil(options.intrinsicDimension * ...
    options.overcompletenessRatio);
% -------------------------------------------------------------------------
if options.IcTKM_embedding_disabled
    optsIcTKM.noSketching = 1;
else    
    if options.IcTKM_embedding_PartialFourier
        optsIcTKM.FJLTtype = 'PartialFFT';    
    elseif options.IcTKM_embedding_PartialDCT
        optsIcTKM.FJLTtype = 'PartialDCT';    
    elseif options.IcTKM_embedding_PartialCirculant
        optsIcTKM.FJLTtype = 'PartialCirculant';
    end
    
    optsIcTKM.m = options.IcTKM_embedding_dimension;
end
% -------------------------------------------------------------------------
if options.IcTKM_runMode_parallel
    optsIcTKM.runMode = 'parallel';
    init_parallel_toolbox(options.IcTKM_runMode_parallel_boost_workers);
elseif options.IcTKM_runMode_sequential
    optsIcTKM.runMode = 'sequential';
end

if options.IcTKM_runMode_useGPU
    if is_gpu_available()        
        init_gpu();
        optsIcTKM.useGPU = 1;
    else
        optsIcTKM.useGPU = 0;
    end
else
    optsIcTKM.useGPU = 0;
end
% -------------------------------------------------------------------------
if options.DataModeSynthetic    
    S = options.SyntheticData_S;
    rho = options.SyntheticData_noiseLevel;      
    flatCoeff = options.SyntheticData_flatCoeff;
    dynamicRange = options.SyntheticData_dynamicRange;
    fixedCoeffDecay = options.SyntheticData_fixedCoeffDecay;
    recThres = options.SyntheticData_recThres;
    %Sample size N is of order eps*K*log(K)
    N = ceil( options.SampleSizeRatio*optsIcTKM.K*log(optsIcTKM.K) ); 
    
    if options.SyntheticData_origDicoRandom
        origDicoType = 'Random';
    elseif options.SyntheticData_origDicoDiracDCT
        origDicoType = 'DiracDCT';
    elseif options.SyntheticData_origDicoHaarDCT
        origDicoType = 'HaarDCT';
    elseif options.SyntheticData_origDicoDiracHadamard
        origDicoType = 'DiracHadamard';
    end   
    
    dataObj = ictkm_synthetic_data(options.ambientDimension, ...
        options.intrinsicDimension, options.overcompletenessRatio, ...
        S, S, N, flatCoeff, dynamicRange, fixedCoeffDecay, ...
        rho, recThres, origDicoType, optsIcTKM.useGPU);
elseif options.DataModeAudio
    S = options.AudioData_S;
    frameLength = options.AudioData_frameLengthInSeconds;
    frameOffset = options.AudioData_frameOffsetInSeconds;
    numFramesPerBlock = options.AudioData_numFramesPerBlock;
    filename = options.AudioData_fileName;
    
    % Sanity check audio input parameters
    if (frameOffset > frameLength)
        error(['IcTKMSimulation: The frame length must be greater ' ...
            'than the frame offset.']);
    end    
    if ~(numFramesPerBlock > 0)
        error(['IcTKMSimulation: The number of frames per block must ' ...
            'be greater than zero.']);
    end
    
    dataObj = ictkm_audio_data(S, filename, numFramesPerBlock, ...
        frameLength, frameOffset, optsIcTKM.nonNegative);
end
% -------------------------------------------------------------------------
if options.useSeed    
    % Reset random number generator to default state and
    % set seed for reproducible results.
    rng(options.randomSeed);
    if options.IcTKM_runMode_parallel
        % Will need this to configure the random generator
        % every time that a worker starts executing in IcTKMCoreParFor.m
        optsIcTKM.randomSeed = options.randomSeed;
    end         
    
    if optsIcTKM.useGPU
        % GPU has its own random generator
        sg = parallel.gpu.RandStream('CombRecursive','NormalTransform',...
            'Inversion','Seed', options.randomSeed);
        parallel.gpu.RandStream.setGlobalStream(sg);
    end
    
    optsIcTKM.useSeed = 1;
else    
    optsIcTKM.useSeed = 0;
end
% -------------------------------------------------------------------------
optsDicoInit = [];
optsDicoInit.useGPU = optsIcTKM.useGPU;
optsDicoInit.d = optsIcTKM.d;
optsDicoInit.K = optsIcTKM.K;
optsDicoInit.nonNegativeConstraint = options.IcTKM_nonNegativeConstraint;
if options.DicoInit_random
    optsDicoInit.type = 'Random';    
elseif options.DicoInit_oracle
    optsDicoInit.type = 'Oracle';
elseif options.DicoInit_1_1_ratio
    optsDicoInit.type = '1:1_ratio';
elseif options.DicoInit_1_4_ratio
    optsDicoInit.type = '1:4_ratio';
end
if options.DataModeSynthetic
    optsDicoInit.odico = dataObj.dico;
end
dinit = ictkm_dico_init(optsDicoInit);
% -------------------------------------------------------------------------
% Call the IcTKM algorithm to perform the simulation
% -------------------------------------------------------------------------
results = ictkm(dinit,dataObj,optsIcTKM);

% =========================================================================
% Auxiliary functions
% =========================================================================
    function options = validate_options(options)
        % -----------------------------------------------------------------
        if ~isfield(options, 'IcTKM_numIter')
            error(['IcTKMSimulation: Field "IcTKM_numIter" is needed '...
                'in the options struct .']);
        end
        if ~isfield(options, 'IcTKM_verbose')
            options.IcTKM_verbose = 0;
        end
        if ~isfield(options, 'IcTKM_nonNegativeConstraint')
            options.IcTKM_nonNegativeConstraint = 0;
        end        
        if isfield(options, 'randomSeed')
            options.useSeed = 1;
        else
            options.useSeed = 0;
        end
        % -----------------------------------------------------------------
        if ~isfield(options, 'ambientDimension')
            error(['IcTKMSimulation: Field "ambientDimension" is needed in the options'...
                ' struct.']);
        end
        if ~isfield(options, 'intrinsicDimension')
            options.intrinsicDimension = options.ambientDimension;
        end
        if ~isfield(options, 'overcompletenessRatio')
            error(['IcTKMSimulation: Field "overcompletenessRatio" is needed in the options'...
                ' struct.']);
        end
        % -----------------------------------------------------------------
        if ~isfield(options, 'IcTKM_embedding_disabled')
            options.IcTKM_embedding_disabled = 0;
            options.IcTKM_embedding_PartialFourier = 0;
            options.IcTKM_embedding_PartialDCT = 0;
            options.IcTKM_embedding_PartialCirculant = 0;
        end
        if ~options.IcTKM_embedding_disabled
            if ~isfield(options, 'IcTKM_embedding_PartialFourier') && ...
                ~isfield(options, 'IcTKM_embedding_PartialDCT') && ...                    
                ~isfield(options, 'IcTKM_embedding_PartialCirculant')
                error(['IcTKMSimulation: Field '...
                    '"IcTKM_embedding_PartialFourier", ' ...
                    '"IcTKM_embedding_PartialDCT", or ', ...                    
                    'IcTKM_embedding_PartialCirculant' ...
                    'is needed in the options struct.']);
            end
            if ~isfield(options, 'IcTKM_embedding_PartialFourier')
                options.IcTKM_embedding_PartialFourier = 0;
            end
            if ~isfield(options, 'IcTKM_embedding_PartialDCT')
                options.IcTKM_embedding_PartialDCT = 0;
            end
            if ~isfield(options, 'IcTKM_embedding_PartialCirculant')
                options.IcTKM_embedding_PartialCirculant = 0;
            end
            
            if ~options.IcTKM_embedding_PartialFourier && ...
                    ~options.IcTKM_embedding_PartialDCT && ...
                    ~options.IcTKM_embedding_PartialCirculant
                % Sets default
                options.IcTKM_embedding_PartialFourier = 1;
            end
            
            
            if ~isfield(options, 'IcTKM_embedding_dimension')
                error(['IcTKMSimulation: Field "IcTKM_embedding_dimension" '...
                    'is needed in the options struct.']);
            end
        end
        % -----------------------------------------------------------------
        if ~isfield(options, 'IcTKM_runMode_parallel') && ...
                ~isfield(options, 'IcTKM_runMode_sequential')
            error(['IcTKMSimulation: At least one field '...
                '"IcTKM_runMode_..." is needed in the options struct.']);
        end
        if ~isfield(options, 'IcTKM_runMode_sequential')
            options.IcTKM_runMode_sequential = 0;
        end
        if ~isfield(options, 'IcTKM_runMode_useGPU')
            options.IcTKM_runMode_useGPU = 0;
        end        
        if ~isfield(options, 'IcTKM_runMode_parallel')
            options.IcTKM_runMode_parallel = 0;
        end        
        if ~options.IcTKM_runMode_parallel && ...
                ~options.IcTKM_runMode_sequential
            % Select default
            options.IcTKM_runMode_sequential = 1;
        end
        if ~isfield(options, 'IcTKM_runMode_parallel_boost_workers')
            options.IcTKM_runMode_parallel_boost_workers = 0;
        end
        % -----------------------------------------------------------------
        if ~isfield(options, 'SampleSizeRatio')
            options.SampleSizeRatio = 1;
        end
        if ~isfield(options, 'DataModeSynthetic') && ...
                ~isfield(options, 'DataModeAudio') && ...
                ~isfield(options, 'DataModeImagePatches') && ...
                ~isfield(options, 'DataModeUser')
            error(['IcTKMSimulation: At least one field "DataMode..." '...
                'is needed in the options struct.']);
        end        
        if ~isfield(options, 'DataModeSynthetic')
            options.DataModeSynthetic = 0;
        end        
        if ~isfield(options, 'DataModeAudio')
            options.DataModeAudio = 0;
        end                
        if ~isfield(options, 'DataModeImagePatches')
            options.DataModeImagePatches = 0;
        end                
        if ~isfield(options, 'DataModeUser')
            options.DataModeUser = 0;
        end
        if options.DataModeSynthetic
            if ~isfield(options, 'SyntheticData_S')
                error(['IcTKMSimulation: Field "SyntheticData_S" is '...
                    'needed in the options struct.']);
            end
            if ~isfield(options, 'SyntheticData_noiseLevel')
                error(['IcTKMSimulation: Field "SyntheticData_noiseLevel" '...
                    'is needed in the options struct.']);
            end
            if ~isfield(options, 'SyntheticData_flatCoeff')
                error(['IcTKMSimulation: Field '...
                    '"SyntheticData_flatCoeff" is needed in the '...
                    'options struct.']);
            end
            if ~isfield(options, 'SyntheticData_recThres')
                error(['IcTKMSimulation: Field "SyntheticData_recThres"'...
                    'is needed in the options struct.']);
            end
            if ~isfield(options,'SyntheticData_origDicoRandom')&&...
                    ~isfield(options,'SyntheticData_origDicoDiracDCT')&&...
                    ~isfield(options,'SyntheticData_origDicoHaarDCT')&&...
                    ~isfield(options,'SyntheticData_origDicoDiracHadamard')
                error(['IcTKMSimulation: At least one field '...
                    '"SyntheticData_origDico..." is needed in the '...
                    'options struct.']);
            end
            if ~isfield(options,'SyntheticData_origDicoRandom')
                options.SyntheticData_origDicoRandom = 0;
            end
            if ~isfield(options,'SyntheticData_origDicoDiracDCT')
                options.SyntheticData_origDicoDiracDCT = 0;
            end
            if ~isfield(options,'SyntheticData_origDicoHaarDCT')
                options.SyntheticData_origDicoHaarDCT = 0;
            end
            if ~isfield(options,'SyntheticData_origDicoDiracHadamard')
                options.SyntheticData_origDicoDiracHadamard = 0;
            end            
            if ~options.SyntheticData_origDicoRandom && ...
                    ~options.SyntheticData_origDicoDiracDCT && ...
                    ~options.SyntheticData_origDicoHaarDCT && ...
                    ~options.SyntheticData_origDicoDiracHadamard
                % Select default
                options.SyntheticData_origDicoRandom = 1;
            end
        elseif options.DataModeAudio            
            if ~isfield(options, 'AudioData_S')
                error(['IcTKMSimulation: Field "AudioData_S" is '...
                    'needed in the options struct.']);
            end
            if ~isfield(options, 'AudioData_frameLengthInSeconds')
                error(['IcTKMSimulation: Field "AudioData_frameLengthInSeconds" is '...
                    'needed in the options struct.']);
            end
            if ~isfield(options, 'AudioData_frameOffsetInSeconds')
                error(['IcTKMSimulation: Field "AudioData_frameOffsetInSeconds" is '...
                    'needed in the options struct.']);
            end          
            if ~isfield(options, 'AudioData_numFramesPerBlock')
                error(['IcTKMSimulation: Field "AudioData_numFramesPerBlock" is '...
                    'needed in the options struct.']);
            end                      
            if ~isfield(options, 'AudioData_fileName')
                error(['IcTKMSimulation: Field "AudioData_fileName" is '...
                    'needed in the options struct.']);
            end            
        elseif options.DataModeImagePatches
            %TODO: Add validation here
        end
        % -----------------------------------------------------------------
        if ~isfield(options, 'DicoInit_random') && ...
                ~isfield(options, 'DicoInit_oracle') && ...
                ~isfield(options, 'DicoInit_1_1_ratio') && ...
                ~isfield(options, 'DicoInit_1_4_ratio')
            error(['IcTKMSimulation: At least one field "DicoInit_..."'...
                'is needed in the options struct.']);
        end
        if ~isfield(options, 'DicoInit_random')
            options.DicoInit_random = 0;
        end
        if ~isfield(options, 'DicoInit_oracle')
            options.DicoInit_oracle = 0;
        end        
        if ~isfield(options, 'DicoInit_1_1_ratio')
            options.DicoInit_1_1_ratio = 0;
        end
        if ~isfield(options, 'DicoInit_1_4_ratio')
            options.DicoInit_1_4_ratio = 0;
        end
        
        if ~options.DicoInit_random && ...
                ~options.DicoInit_oracle && ...
                ~options.DicoInit_1_1_ratio && ...
                ~options.DicoInit_1_4_ratio
            % Select default
            options.DicoInit_random = 1;
        end
        % -----------------------------------------------------------------
    end
end
