function ictkm_synthetic_simulation(...
                                        randomSeed, ...
                                        ambientDimension, ...
                                        intrinsicDimension, ...
                                        sampleSizeRatio, ...
                                        IcTKM_compressionRatio, ...
                                        useLowerSparsityLevel, ...
                                        IcTKM_numIterations,...
                                        IcTKM_embedding_PartialFourier, ...
                                        IcTKM_embedding_PartialDCT, ...
                                        IcTKM_embedding_PartialCirculant,...
                                        verboseAlgorithm,...
                                        useParallelization,...
                                        useGpuAcceleration,...
                                        resultsFilename...
                                    )
% -------------------------------------------------------------------------
% Runs a dictionary learning synthetic data simulation with IcTKM.
% -------------------------------------------------------------------------
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compressed Dictionary Learning Toolbox
% Copyright (C) Flavio Teixeira 2018
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, 
% Boston, MA  02110-1301, USA.    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                                

% =========================================================================
% Display simulation parameters
% =========================================================================
disp('ictkm_synthetic_simulation parameters')
disp([sprintf('\t') 'ambientDimension: ' num2str(ambientDimension) ]);
disp([sprintf('\t') 'intrinsicDimension: ' num2str(intrinsicDimension) ]);
disp([sprintf('\t') 'sampleSizeRatio: ' num2str(sampleSizeRatio) ]);
disp([sprintf('\t') 'IcTKM_compressionRatio: ' num2str(IcTKM_compressionRatio) ]);
disp([sprintf('\t') 'useLowerSparsityLevel: ' num2str(useLowerSparsityLevel) ]);
disp([sprintf('\t') 'IcTKM_numIterations: ' num2str(IcTKM_numIterations) ]);
disp([sprintf('\t') 'IcTKM_embedding_PartialFourier: ' ...
    num2str(IcTKM_embedding_PartialFourier) ]);
disp([sprintf('\t') 'IcTKM_embedding_PartialDCT: ' ...
    num2str(IcTKM_embedding_PartialDCT) ]);
disp([sprintf('\t') 'IcTKM_embedding_PartialCirculant: ' ...
    num2str(IcTKM_embedding_PartialCirculant) ]);

optsIcTKMSimulation=[];

% =========================================================================
% Set IcTKM misc configurations
% =========================================================================
optsIcTKMSimulation.IcTKM_numIter = IcTKM_numIterations;
optsIcTKMSimulation.IcTKM_verbose = verboseAlgorithm;
optsIcTKMSimulation.IcTKM_storeRecCorr = 1;
optsIcTKMSimulation.IcTKMHaltOnPerfectRecovery = 0;
optsIcTKMSimulation.randomSeed = randomSeed;

% =========================================================================
% Sets the IcTKM Run mode
% =========================================================================
optsIcTKMSimulation.IcTKM_runMode_parallel = useParallelization;
optsIcTKMSimulation.IcTKM_runMode_parallel_boost_workers = 0;
optsIcTKMSimulation.IcTKM_runMode_useGPU = useGpuAcceleration;

% =========================================================================
% Create initialization for the IcTKM
% =========================================================================
optsIcTKMSimulation.DicoInit_random = 1;

% =========================================================================
% Define the problem size
% =========================================================================
optsIcTKMSimulation.ambientDimension = ambientDimension;
optsIcTKMSimulation.intrinsicDimension = intrinsicDimension;
optsIcTKMSimulation.overcompletenessRatio = 3/2;

% =========================================================================
% Create data set for learning the dictionary
% =========================================================================
optsIcTKMSimulation.SampleSizeRatio = sampleSizeRatio;
optsIcTKMSimulation.DataModeSynthetic = 1;

% =========================================================================
% Chooses the dictionary for generating the data
% =========================================================================
optsIcTKMSimulation.SyntheticData_origDicoRandom = 1;

if useLowerSparsityLevel
    optsIcTKMSimulation.SyntheticData_S = 4;
else
    optsIcTKMSimulation.SyntheticData_S = ceil( ...
        sqrt(optsIcTKMSimulation.intrinsicDimension)/2 );
end

optsIcTKMSimulation.SyntheticData_dynamicRange = 4;

optsIcTKMSimulation.SyntheticData_fixedCoeffDecay = 0;
optsIcTKMSimulation.SyntheticData_flatCoeff = 0;
optsIcTKMSimulation.SyntheticData_noiseLevel = ...
    1/( 2*sqrt(optsIcTKMSimulation.ambientDimension) );

optsIcTKMSimulation.SyntheticData_recThres = 0.99;

if (~IcTKM_embedding_PartialFourier && ...
        ~IcTKM_embedding_PartialDCT && ...
        ~IcTKM_embedding_PartialCirculant)
    
    optsIcTKMSimulation.IcTKM_embedding_disabled = 1;% Disable JL embedding
else
    optsIcTKMSimulation.IcTKM_embedding_disabled = 0;
    optsIcTKMSimulation.IcTKM_embedding_PartialFourier = ...
        IcTKM_embedding_PartialFourier;
    optsIcTKMSimulation.IcTKM_embedding_PartialDCT = ...
        IcTKM_embedding_PartialDCT;
    optsIcTKMSimulation.IcTKM_embedding_PartialCirculant=...
        IcTKM_embedding_PartialCirculant;
    optsIcTKMSimulation.IcTKM_embedding_dimension = ...
        ceil(optsIcTKMSimulation.ambientDimension/IcTKM_compressionRatio);
end

% =========================================================================
% Run the IcTKM simulation
% =========================================================================
simResults = ictkm_simulation(optsIcTKMSimulation);

% =========================================================================
% Collect simulation results and save to file
% =========================================================================
rcorr_per_iter = []; %#ok<NASGU>
percent_recovered_atoms_vec = []; %#ok<NASGU>

if optsIcTKMSimulation.IcTKM_storeRecCorr
    rcorr_per_iter = simResults.rcorrIter; %#ok<NASGU>
else
    percent_recovered_atoms_vec = simResults.recovery_percent; %#ok<NASGU>
end

avg_recovery_time = simResults.avgTimePerIter; %#ok<NASGU>

save(resultsFilename , ...
    'rcorr_per_iter', ...
    'percent_recovered_atoms_vec', ...    
    'avg_recovery_time');
end
