function dico_audio = ictkm_synthesize_dictionary_to_audio(dataObj, dnew)
% -------------------------------------------------------------------------
% Synthesize a learned dictionary to audio.
% -------------------------------------------------------------------------
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compressed Dictionary Learning Toolbox
% Copyright (C) Flavio Teixeira 2018
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor,
% Boston, MA  02110-1301, USA.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
useVocoder = 0;

d = size(dnew, 1);
K = size(dnew, 2);

fs = dataObj.samplingFrequency;

if (dataObj.useSTFT)
    white_noise_duration = .5;
    white_noise = rand(ceil(fs*white_noise_duration),1)-.5;
    
    frameOffset = floor(d/dataObj.numFramesPerBlock);
    
    % Digital filter parameters
    fir_n = frameOffset; % Filter order
    fir_n = round( (fir_n - 2) / 2 ) * 2 + 2; % round to nearest even integer
    f = (0:1/(frameOffset-1):1)'; % Frequency grid for designing the filter
    
    dico_audio = [];
    for i=1:K
        frameIdx=1;
        
        for j=1:dataObj.numFramesPerBlock
            % =============================================================
            % Synthetize audio
            % =============================================================
            % Design FIR filter with frequency response defined by each atom
            b_fir = fir2(fir_n, f, dnew(frameIdx:(frameIdx+frameOffset-1),i));
            % Feed filter with white noise to synthetize atom's "sound"
            atom_audio = filter(b_fir, 1, white_noise);
            
            % =============================================================
            % Normalize audio (-1 to +1 range)
            % =============================================================
            atom_audio = atom_audio./max(abs(atom_audio(:)))*(1-(2^-(24-1)));
            
            % =============================================================
            % Update dictionary with newly synthesized atom's audio
            % =============================================================
            dico_audio = [dico_audio; atom_audio];
            
            frameIdx = frameIdx + frameOffset;
        end
    end
else
    if useVocoder
        % Atom audio duration in seconds
        atom_duration = d/fs;
        % We want duration to be at least 1 second for proper listening
        duration_sec = 1;
        % Parameters for phase vocoder
        r = atom_duration/duration_sec; % dilation/compression ratio
        window_length = fs*(100e-3); % 100 miliseconds worth of samples
        window_length = round( (window_length - 2) / 2 )...
            * 2 + 2; % round to nearest even integer
    end
    
    dnew = sort_dictionary_by_fundamental_frequency(dnew, d, K);
    
    dico_audio = [];
    for i=1:K
        atom_audio = dnew(:,i);
        
        if dataObj.useWindowing
            % Undo windowing
            w = hamming(dataObj.frameLength, 'periodic');
            atom_audio = atom_audio./w;
        end
        
        if useVocoder
            % Use phase-vocoder to stretch/dilate audio at ratio r
            atom_audio = pvoc(atom_audio, r, window_length);
        end
        
        % =================================================================
        % Normalize audio (-1 to +1 range)
        % =================================================================
        atom_audio = atom_audio./...
            max(abs(atom_audio(:)))*(1-(2^-(24-1)));
        
        % =================================================================
        % Update dictionary with normalized atom's audio (and add half a
        % second pause from one atom to the next)
        % =================================================================
        dico_audio = [dico_audio; atom_audio; zeros(fs/2, 1)];
    end
end

    function dico_sorted = sort_dictionary_by_fundamental_frequency(dico, d, K)
        d_2 = ceil(d/2);
        spectr = zeros(d_2,K);
        max_indices = zeros(K, 1);
        for ii=1:K
            X_mags = abs(fft(dico(:,ii)));
            
            spectr(:,ii) = X_mags(1:d_2);
            
            [~, indx] = max( spectr(:,ii) );
            max_indices(ii) = indx;
        end
        
        [~,I] = sort(max_indices);
        
        dico_sorted = dico(:,I);                
    end
end
